package com.mdtexfx.io;

import java.util.Set;

import static com.mdtexfx.io.MediaType.TEXT_MARKDOWN;

/**
 * Responsible for associating file extensions with {@link MediaType} instances.
 */
public enum MediaTypeExtensions {
  MARKDOWN( TEXT_MARKDOWN, Set.of(
      "md", "markdown", "mdown", "mdtxt", "mdtext", "mdwn", "mkd", "mkdown",
      "mkdn", "text", "txt" ) );

  private final MediaType mMediaType;
  private final Set<String> mExtensions;

  /**
   * Constructs an association of file name extensions to a single {@link
   * MediaType}.
   *
   * @param mediaType  The {@link MediaType} to associate with the given
   *                   file name extensions.
   * @param extensions The file name extensions used to lookup a corresponding
   *                   {@link MediaType}.
   */
  MediaTypeExtensions(
      final MediaType mediaType, final Set<String> extensions ) {
    assert mediaType != null;
    assert extensions != null;
    assert !extensions.isEmpty();

    mMediaType = mediaType;
    mExtensions = extensions;
  }

  /**
   * Returns the {@link MediaType} associated with the given file name
   * extension. The extension must not contain a period.
   *
   * @param extension File name extension, case insensitive, {@code null}-safe.
   * @return The associated {@link MediaType} as defined by IANA.
   */
  static MediaType getMediaType( final String extension ) {
    final var sanitized = sanitize( extension );

    for( final var mediaType : values() ) {
      if( mediaType.isType( sanitized ) ) {
        return mediaType.getMediaType();
      }
    }

    return MediaType.UNDEFINED;
  }

  private boolean isType( final String extension ) {
    return mExtensions.contains( sanitize( extension ) );
  }

  private static String sanitize( final String extension ) {
    return extension == null ? "" : extension.toLowerCase();
  }

  private MediaType getMediaType() {
    return mMediaType;
  }
}
